 /*****************************************************************************
 *   Copyright (C) 2004 by Michael Schulze                                    *
 *   mike.s@genion.de                                                         *
 *                                                                            *
 *  The code contained in this file is free software; you can redistribute    *
 *  it and/or modify it under the terms of the GNU Lesser General Public      *
 *  License as published by the Free Software Foundation; either version      *
 *  2.1 of the License, or (at your option) any later version.                *
 *                                                                            *
 *  This file is distributed in the hope that it will be useful,              *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 *  Lesser General Public License for more details.                           *
 *                                                                            *
 *  You should have received a copy of the GNU Lesser General Public          *
 *  License along with this code; if not, write to the Free Software          *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 *                                                                            *
 *  iTunes and iPod are trademarks of Apple                                   *
 *                                                                            *
 *  This product is not supported/written/published by Apple!                 *
 *****************************************************************************/

#ifndef ITUNESDBITUNESDBLISTENER_H
#define ITUNESDBITUNESDBLISTENER_H

#include <qstring.h>
#include "track.h"
#include "playlist.h"
#include "playlistitem.h"

namespace itunesdb {

/**
 * The methods in this interface will be called by ITunesDBParser during the parsing process.
 * @author Michael Schulze
*/
class ItunesDBListener {

public:

    virtual ~ItunesDBListener() {}

    /**
     * Creates a new Track. Override this method if you need your own Track implementation.
     * The returned pointer will be handed over to the @c handleTrack() method during the
     * parse process.
     */
    virtual Track * createNewTrack();

    /**
     * Creates a new Playlist. Override this method if you need your own Playlist implementation.
     * The returned pointer will be handed over to the @c handlePlaylist() and @c handleOTGPlaylist() 
     * methods during the parse process.
     */
    virtual Playlist * createNewPlaylist();

    /**
     * gets called if an error occured during the parse process.
     * The parse process ends after a call to this method.
     */
    virtual void handleError( const QString& message) = 0;

    /**
     * gets called if the parser finds a mhsd block
     */
    virtual void handleDataSet(Q_UINT32 type) = 0;

    /**
     * sets the number of tracks (information from the itunesdb file)
     */
    virtual void setNumTracks( Q_UINT32 numtracks) = 0;

    /**
     * sets the number of playlists (information from the itunesdb file)
     */
    virtual void setNumPlaylists( Q_UINT32 numplaylists) = 0;

    /**
     * Handles a given Playcount entry
     */
    virtual void handlePlaycount( Q_UINT32 idx, Q_UINT32 lastplayed, Q_UINT32 stars, Q_UINT32 count, Q_UINT32 bookmark) = 0;

    /**
     * Handles the given on the go playlist. The pointer was created with the createNewPlaylist Method.
     * The implementor is the owner thus responsible for managing the pointer.
     */
    virtual void handleOTGPlaylist( Playlist * playlist ) = 0;

    /**
     * handles the given playlist. The pointer was created with the @c createNewPlaylist Method.
     * The implementor is the owner thus responsible for managing the pointer.
     */
    virtual void handlePlaylist( Playlist * playlist ) = 0;

    /**
     * handles the given track. The pointer was created with the @c createNewTrack Method.
     * The implementor is the owner thus responsible for managing the pointer.
     */
    virtual void handleTrack( Track * track) = 0;

    /**
     * gets called at the beginning of the parse process.
     */
    virtual void parseStarted() = 0;

    /**
     * parsing process done. No calls to the methods above will happen from the parser.
     */
    virtual void parseFinished() = 0;
};

}

#endif
